package vazkii.quark.world.world;

import java.util.Random;

import net.minecraft.block.state.IBlockState;
import net.minecraft.init.Blocks;
import net.minecraft.server.MinecraftServer;
import net.minecraft.util.Mirror;
import net.minecraft.util.ResourceLocation;
import net.minecraft.util.Rotation;
import net.minecraft.util.math.BlockPos;
import net.minecraft.util.math.ChunkPos;
import net.minecraft.world.World;
import net.minecraft.world.chunk.IChunkProvider;
import net.minecraft.world.gen.IChunkGenerator;
import net.minecraft.world.gen.structure.StructureBoundingBox;
import net.minecraft.world.gen.structure.template.PlacementSettings;
import net.minecraft.world.gen.structure.template.Template;
import net.minecraft.world.gen.structure.template.TemplateManager;
import net.minecraftforge.fml.common.IWorldGenerator;
import vazkii.quark.world.feature.NetherFossils;

public class NetherFossilGenerator implements IWorldGenerator {

	private static final ResourceLocation STRUCTURE_SPINE_01 = new ResourceLocation("fossils/fossil_spine_01");
	private static final ResourceLocation STRUCTURE_SPINE_02 = new ResourceLocation("fossils/fossil_spine_02");
	private static final ResourceLocation STRUCTURE_SPINE_03 = new ResourceLocation("fossils/fossil_spine_03");
	private static final ResourceLocation STRUCTURE_SPINE_04 = new ResourceLocation("fossils/fossil_spine_04");
	private static final ResourceLocation STRUCTURE_SKULL_01 = new ResourceLocation("fossils/fossil_skull_01");
	private static final ResourceLocation STRUCTURE_SKULL_02 = new ResourceLocation("fossils/fossil_skull_02");
	private static final ResourceLocation STRUCTURE_SKULL_03 = new ResourceLocation("fossils/fossil_skull_03");
	private static final ResourceLocation STRUCTURE_SKULL_04 = new ResourceLocation("fossils/fossil_skull_04");
	private static final ResourceLocation[] FOSSILS = new ResourceLocation[] {STRUCTURE_SPINE_01, STRUCTURE_SPINE_02, STRUCTURE_SPINE_03, STRUCTURE_SPINE_04, STRUCTURE_SKULL_01, STRUCTURE_SKULL_02, STRUCTURE_SKULL_03, STRUCTURE_SKULL_04};

	@Override
	public void generate(Random random, int chunkX, int chunkZ, World world, IChunkGenerator chunkGenerator, IChunkProvider chunkProvider) {
		if(world.provider.isNether() && random.nextInt(NetherFossils.chance) == 0) {
			int x = chunkX * 16 + random.nextInt(16) + 8;
			int z = chunkZ * 16 + random.nextInt(16) + 8;
			int y = 40;
			BlockPos pos = new BlockPos(x, y, z);
			IBlockState stateAt = null;
			do {
				stateAt = world.getBlockState(pos);
				pos = pos.down();
			} while(stateAt.getBlock().isAir(stateAt, world, pos) && pos.getY() > 0);
			
			if(stateAt.getBlock() == Blocks.LAVA)
				generateFossil(world, random, pos.up(random.nextInt(2)));
		}
	}
	
	// stolen from WorldGenFossils
	private void generateFossil(World world, Random random, BlockPos pos) {
		MinecraftServer minecraftserver = world.getMinecraftServer();
		Rotation[] arotation = Rotation.values();
		Rotation rotation = arotation[random.nextInt(arotation.length)];
		int i = random.nextInt(FOSSILS.length);
		TemplateManager templatemanager = world.getSaveHandler().getStructureTemplateManager();
		Template template = templatemanager.getTemplate(minecraftserver, FOSSILS[i]);
		ChunkPos chunkpos = new ChunkPos(pos);
		StructureBoundingBox structureboundingbox = new StructureBoundingBox(chunkpos.getXStart(), 0, chunkpos.getZStart(), chunkpos.getXEnd(), 256, chunkpos.getZEnd());
		PlacementSettings placementsettings = (new PlacementSettings()).setRotation(rotation).setBoundingBox(structureboundingbox).setRandom(random);
		BlockPos blockpos = template.transformedSize(rotation);

		BlockPos blockpos1 = template.getZeroPositionWithTransform(pos, Mirror.NONE, rotation);
		placementsettings.setIntegrity(1F);
		template.addBlocksToWorld(world, blockpos1, placementsettings, 20);
	}

}
